@@ -2,9 +2,10 @@ import unittest
 
 from fabric.api import env
 from fabric import state
+from fabric.network import disconnect_all
 
 from fabtest.vbox import VirtualBox
-from fabtest.utils import close_fabric_connections
+from fabtest.utils import force_ssh_reconnect
 
 class VirtualBoxTest(unittest.TestCase):
     vm_name = 'Squeeze' # name or uuid of VirtualBox VM
@@ -17,12 +18,16 @@ class VirtualBoxTest(unittest.TestCase):
     headless = True
 
     def setUp(self):
+        self._snapshots = []
         self.box = VirtualBox(self.vm_name, self.vbox_command)
         self.activate_snapshot(self.snapshot)
 
     def tearDown(self):
-        if not self.keep_box:
-            self.box.stop()
+        if self.keep_box:
+            return
+        self.box.stop()
+        for name in self._snapshots:
+            self.box.snapshot('delete', name)
 
     def activate_snapshot(self, name):
         assert self.box.snapshot_exists(name), 'Snapshot "%s" does not exist' % name
@@ -30,6 +35,13 @@ class VirtualBoxTest(unittest.TestCase):
         self.box.snapshot('restore', name)
         self.box.start(self.headless)
 
+    def take_test_snapshot(self, name):
+        """
+        Creates temporary snapshot that will be deleted after test.
+        """
+        self._snapshots.append(name)
+        self.box.snapshot_take(name)
+
 
 class FabTest(VirtualBoxTest):
     host = 'root@127.0.0.1:2222'
@@ -37,18 +49,33 @@ class FabTest(VirtualBoxTest):
     key_filename = None
 
     def setUp(self):
-        super(FabTest, self).setUp()
-        close_fabric_connections()
-        self.previous_env = state._AttributeDict(env)
         self.setup_env()
+        disconnect_all()
+        self.previous_env = state._AttributeDict(env)
+        super(FabTest, self).setUp()
 
     def tearDown(self):
-        close_fabric_connections()
+        disconnect_all()
         env.update(self.previous_env)
         super(FabTest, self).tearDown()
 
     def setup_env(self):
         env.hosts = [self.host]
         env.password = self.password
+        env.host_string = self.host
         env.key_filename = self.key_filename
+        env.timeout = 1 # local virtualbox connections should be fast
         env.disable_known_hosts = True
+
+    def take_test_snapshot(self, name, reconnect=True):
+        super(FabTest, self).take_test_snapshot(name)
+        if reconnect:
+            force_ssh_reconnect()
+
+    def activate_test_snapshot(self, name, reconnect=True):
+        """
+        Activates test snapshot.
+        """
+        self.activate_snapshot(name)
+        if reconnect:
+            force_ssh_reconnect()
